float3 main(

    in float3 normal : TEXCOORD0,   // Input from the vertex shader
    in float3 eyeVec : TEXCOORD01,  // Eye vector computed in the vertex program

    uniform samplerCUBE aCubeMapTex // cubemap texture of the environment

) : COLOR
{
    // The texcoords are interpolated along the triangle.
    // Since the interpolation may produce vector with non-unit length,
    // we have to renormalize these vectors.
    normal = normalize(normal);
    eyeVec = normalize(eyeVec);

    // compute the reflected vector, use the reflect() function of the Cg library
    float3 dir = reflect(eyeVec, normal);

    // sample the environment map texture along the reflected vector 
    float3 color = texCUBE(aCubeMapTex, dir);
    
    // setup the base color of the water
	float3 baseCol = float3(0.2, 1, 0.85) * 0.15;

    // combine the base color and the reflected color to create nice looking water :-)
    float3 result = baseCol + color;

	return result;
}
