void main(
	uniform float4x4 aMVP,
    uniform float4x4 aMVI,      // Inverse Modelview matrix
	uniform float time,          // global time
	uniform float amplitude,     // Wave amplitude
	uniform float frequency,     // wave frequency
	in float4 aPos : POSITION,   // position as passed by glVertex3f() method

	out float4 oPos : POSITION,
    out float3 normal : TEXCOORD0, // we pass the computed normal to the fragment program
    out float3 eyeVec : TEXCOORD1  // pass the computed eye vector to the fragment program
)
{

    //-------------------------------------------------------------------------
    // EXERCISE 9.4.a)

	// To simulate water movement (just very smple one)
	// we do combine trigonometric functions which makes 
	// the vertex's z-coordinate oscillates.

	// The phase of the wave is derived directly from the xy coordinates of the wave vertex
	float2 phase = aPos.xy;

	// Compute the oscilation based on frequency, time, phase and amplitude
	float2 dz = sin(phase + time * frequency) * amplitude;

	// Set the vertex z-coordinate to the new value
	aPos.z = dz.x * dz.y;
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // EXERCISE 9.4.b)

    // We compute now the normal based on the numerical derivate approximation
    // as given in the exercise. Since we have alread compute f(x) we need
    // only to compute f(x+d).

    // Let our d vector be just some 2d vector
    float2 d = float2(0.25, 0.25);

    // compute f(x+d)
    float2 dz2 = sin(phase + time * frequency + d) * amplitude;
    
    // compute the derivative
    float2 fprime = (dz2 - dz) / length(d);

    // setup the normal and normalize it
    normal = float3(fprime.x, fprime.y, 1);
    normal = normalize(normal);

    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // EXERCISE 9.4.c)

    // First we need to calculate the eye point in world coordinates. To do this
    // we need to multiply the inverse of the modelview matrix. Why that ?
    // The modelviewmatrix do specify the transformation of a point in the 
    // model space (or local space) into the camera space. In our case the model 
    // space is the world space, as our water surface is defined in world coordinates.
    // Hence the inverse modelview matrix do specify the transformation 
    // from the viewspace (local space of the camera) into the world space.
    // Since the camera position in view space is (0,0,0,1) we can compute the world
    // positionm of the camera point.
    
    float4 eyePos = mul(aMVI, float4(0,0,0,1));
    
    // perform conversion from the homogeneous space
    eyePos.xyz /= eyePos.w;

    // compute eye vector, the vector pointing from the eye to the vertex
    eyeVec = normalize(aPos.xyz - eyePos.xyz);

    //-------------------------------------------------------------------------

	// calculate the vertex position based on the modelview matrix
	oPos = mul(aMVP, aPos);
}
