
#ifndef __SPLINE_H__
#define __SPLINE_H__

#include <string>
#include <vector>
#include "nv_math/nv_math.h"

class Spline {
public:

	// Constructor & Destructor	
	Spline() : num_points(0), length(0.0){}
	~Spline() {}

    // load spline from file	
	bool loadFromFile(const std::string& name, bool loop = true);

    /**
    * Create spline from points.
    * @param points List of points describing the spline
    * @param loop Set to true to create continious spline (last point is equal to the first one)
    **/
	void createFromPoints(const std::vector<vec3>& points, bool loop = true);

	// Operators to get points
	vec3 operator()(float t);
	vec3 operator()(float t,float speed);
	
private:
	
	void create(const std::vector<vec3>& points, bool loop = true);

    // nubmer of available points	
	int num_points;

    // length of the complete spline curve
	float length;

    // points to parameterise the spline
	std::vector<vec3> params;
};

#endif 
