#include "stdafx.h"
#include "control_and_initialization.h"
#include "variables.h"


//-----------------------------------------------------------------------
vec2 gCameraRotation = vec2(-10, /*182*/10);
bool gbDrawDebugCoordSystem = false;
int gCurrentFrameNumber = 0;
vec3 gCameraPos = vec3(0, 0, 0.5);
vec3 gCameraLookAt = vec3(0, 0.2, -0.4);
bool gbInMotion = false;
vec2 gTrackStartPoint, gStartCameraRotation;
//const vec3 CAMERA_UP = cross(vec3(0, -CAMERA_RADIUS, gCameraCenter.z) - gCameraCenter, vec3(1, 0, 0));
const vec3 CAMERA_UP = vec3(0, 0, 1);
typedef char t_byte;
const int ENV_MAP_SIZE = 512;
Spline gCameraSpline;

//-----------------------------------------------------------------------
void updateCamera()
{
    // compute current time
    float time = gCurrentFrameNumber * (1.0/30.0);

    // compute new camera position
    gCameraPos = gCameraSpline(time, 1.5);
}


//-----------------------------------------------------------------------
void setupCamera()
{
	GLint vpExt[4];
	glGetIntegerv(GL_VIEWPORT, vpExt);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(CAMERA_FOV, vpExt[2] / (float)vpExt[3], 0.1, 1000);
	tstGLError();

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(
        gCameraPos.x, gCameraPos.y, gCameraPos.z,
		//0, -CAMERA_RADIUS, gCameraCenter.z, 
		//gCameraCenter.x, gCameraCenter.y, gCameraCenter.z, 
        gCameraLookAt.x, gCameraLookAt.y, gCameraLookAt.z,
		CAMERA_UP.x, CAMERA_UP.y, CAMERA_UP.z
		);
	glRotatef(gCameraRotation.y, 1, 0, 0);
	glRotatef(gCameraRotation.x, 0, 0, 1);
	tstGLError();

	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();


    // load camera spline
    gCameraSpline.loadFromFile("resource/camera.spline", true);
}

//-----------------------------------------------------------------------
void kbdFunc(unsigned char aKey, int aX, int aY)
{
	switch(aKey)
	{
		case 27:
			exit(0);
			break;

		case 'd':
		case 'D':
			gbDrawDebugCoordSystem = !gbDrawDebugCoordSystem;
			break;
		case 'w':
		case 'W':
			gCameraPos.z += 0.1;
			break;
		case 's':
		case 'S':
			gCameraPos.z -= 0.1;
			break;
	}
}


//-----------------------------------------------------------------------
void mouseFunc(int aButton, int aState, int aX, int aY)
{
	vec2 curMousePos(aX, aY);
	if(aButton == GLUT_LEFT_BUTTON && aState == GLUT_DOWN)
	{
		gbInMotion = true;
		gTrackStartPoint = curMousePos;
		gStartCameraRotation = gCameraRotation;
	}

	if(aButton == GLUT_LEFT_BUTTON && aState == GLUT_UP)
		gbInMotion = false;
}

//-----------------------------------------------------------------------
void mouseMotionFunc(int aX, int aY)
{
	vec2 curMousePos(aX, aY);
	if(gbInMotion)
		gCameraRotation = gStartCameraRotation + (curMousePos - gTrackStartPoint) / 6.;
}

//-----------------------------------------------------------------------
void timerFunc(int aValue)
{
	glutPostRedisplay();
	glutTimerFunc(1000 / 30, timerFunc, 1);
	gCurrentFrameNumber++;
}


//-----------------------------------------------------------------------
void loadTextures()
{
	//Load the sky textures
	const char* skyFaceNames[] = {"top", "bottom", "front", "back", "right", "left"};
	const int cubeMapFaceTargets[] = {
		GL_TEXTURE_CUBE_MAP_POSITIVE_Z, GL_TEXTURE_CUBE_MAP_NEGATIVE_Z,
		GL_TEXTURE_CUBE_MAP_POSITIVE_Y, GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
		GL_TEXTURE_CUBE_MAP_POSITIVE_X, GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
	};

	glGenTextures(1, &ghEnvMapTexture);
	glBindTexture(GL_TEXTURE_CUBE_MAP, ghEnvMapTexture);
	for(int nFace = 0; nFace < 6; nFace++)
	{
		std::string fName = std::string("resource/") + skyFaceNames[nFace] + ".raw";
		std::basic_ifstream<t_byte> rawFile(fName.c_str(), std::ios_base::binary | std::ios_base::in);
		ErrorIf(rawFile.bad());

		rawFile.seekg(0, std::ios_base::end);
		int fileSize = (int)rawFile.tellg();
		rawFile.seekg(0, std::ios_base::beg);
		ErrorIf(fileSize != ENV_MAP_SIZE * ENV_MAP_SIZE * 3);

		std::vector<t_byte> data(fileSize);
		rawFile.read(&data.front(), fileSize);


		gluBuild2DMipmaps(
			cubeMapFaceTargets[nFace], GL_RGB, ENV_MAP_SIZE, 
			ENV_MAP_SIZE, GL_RGB, GL_UNSIGNED_BYTE, &data.front()
			);
		tstGLError();
	}

	tstGLError();

	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	tstGLError();

}

//-----------------------------------------------------------------------
void loadPrograms()
{
	// setup sphere shaders
	gSphereVertexProg.setFilename("resource/sphere_vert.fx");
	gSphereFragmentProg.setFilename("resource/sphere_frag.fx");
	gSphereVertexProg.setup();
	gSphereFragmentProg.setup();

	// setup water sahders
	gWaterVertexProg.setFilename("resource/water_vert.fx");
	gWaterFragmentProg.setFilename("resource/water_frag.fx");
	gWaterVertexProg.setup();
	gWaterFragmentProg.setup();
}

//-----------------------------------------------------------------------
void intialize(int &argc, char* argv[])
{
	glutInit(&argc, argv);
	glutInitWindowSize(640, 480);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutCreateWindow("Water simulation");
	tstGLError();

	loadTextures();
	loadPrograms();

    // just some lighting so that it does not get boring
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glLightfv(GL_LIGHT0, GL_POSITION, vec3(0.2, 0.2, 1.0).vec_array);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, vec3(0.7, 0.5, 0.2).vec_array);
    glLightfv(GL_LIGHT0, GL_AMBIENT, vec3(0.01, 0.01, 0.01).vec_array);


	glutTimerFunc(1000 / 30, timerFunc, 1);
	glutKeyboardFunc(kbdFunc);

	glutMouseFunc(mouseFunc);
	glutMotionFunc(mouseMotionFunc);
}
